/*
A simple utility for load testing TM1 server
Fill the xml testing profile sample, then run
java tm1_stress_test name_of_your_profile_file.xml

Author: Yuri Kudryavcev, mail@ykud.com

Current Version: 0.3

Version History: 
0.3 - 08/11/2012 - @ykud: 
changed reading to getViewArrayValueByRange, for some reason debug printing read values 
didn't work otherwise
changed packaging to include api's from 9.5.1, 9.5.2 and 10

0.2 - 09/09/2012 - @ykud: 
Added seconds_between_actions parameter to include random sleeping periods
Added proper object cleaning (view and subset destroy), stopping memory explosion in writing tests

0.1 - sometime in 2010 - @ykud:
First working draft.
*/
import com.applix.tm1.*;
import java.io.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.ArrayList; 
import java.util.Random;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

class CloseLogging extends Thread
/* Simple hook to correctly close log file on CTRL + C */
{
	BufferedWriter out;
	CloseLogging (BufferedWriter out)
	{
		this.out = out;
	}
	public void run() 
	{
		try
		{
			out.close();
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}
	
}

class UserThread extends Thread
/* Emulating user behaviour -- constructing a random view and reading some data from given cubesToRead list, 
writing data to a random number of cells in cubesToWrite list */
{
	String AdminHost; 
	String TM1ServerName; 
	String user; 
	String password;
	String[] cubesToRead;
	String[] cubesToWrite;
	String[] writeRestrictedDimensions;
	ArrayList<String>[] writeRestrictedDimensionElements;
	String[] readRestrictedDimensions;
	ArrayList<String>[] readRestrictedDimensionElements;
	boolean debug;
	BufferedWriter logFile;
	int sessionId;
	DateFormat dateFormat;
	int MAX_NUMBER_TO_WRITE;
	int secondsBetweenActions;
	UserThread(String AdminHost, String TM1ServerName, String user, String password, String[] cubesToRead, String[] cubesToWrite, String[] writeRestrictedDimensions,	ArrayList<String>[] writeRestrictedDimensionElements, String[] readRestrictedDimensions,	ArrayList<String>[] readRestrictedDimensionElements, BufferedWriter logFile, int sessionId, boolean debug, int secondsBetweenActions)
	{
		this.AdminHost = AdminHost;
		this.TM1ServerName = TM1ServerName;
		this.user = user;
		this.password = password;
		this.cubesToRead = cubesToRead;
		this.cubesToWrite = cubesToWrite;
		this.writeRestrictedDimensions = writeRestrictedDimensions;
		this.writeRestrictedDimensionElements = writeRestrictedDimensionElements;
		this.readRestrictedDimensions = readRestrictedDimensions;
		this.readRestrictedDimensionElements = readRestrictedDimensionElements;
		this.logFile = logFile;
		this.sessionId = sessionId;
		this.dateFormat = new SimpleDateFormat("HH:mm:ss");
		this.debug = debug;
		this.MAX_NUMBER_TO_WRITE = 1000;
		this.secondsBetweenActions = secondsBetweenActions;
	}
	public void writeToLog (String cubeName, String message, float duration)
	{
		try
		{	
			Date date = new Date(); 
			logFile.write(sessionId + ";" + user + ";" + dateFormat.format(date) +";" + cubeName + ";" + message + ";" + duration + ";\n");
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		} 
	}
	public void readDataFromCube(TM1Cube targetCube, Random r, TM1Server conn)
	/*Construct a random view and read data from it*/
	{
		TM1Val cellCoordinates;
		TM1Element cellCoordinate;
		TM1Subset sub;
		TM1Val titleSubsetArr;
		TM1Val columnSubsetArr;
		TM1Val rowSubsetArr;
		TM1Subset titleSubset[];
		TM1View vwToRead;
		TM1Val valueArr;
		TM1Val val;
		TM1Query resultQuery;
		TM1Val result;
		int dimCount;
		int elemCount;
		long startTime;
		long endTime;
		boolean restrictedDimension;
		//Create and read a Cube view
		dimCount = targetCube.getDimensionCount().getInt();
		rowSubsetArr = TM1Val.makeArrayVal(1);
		columnSubsetArr = TM1Val.makeArrayVal(1);
		titleSubsetArr = TM1Val.makeArrayVal(dimCount -2);
		titleSubset = new TM1Subset[dimCount - 2];
		System.out.println ("Reading from " + targetCube.getName().getString());
		//Picking some elements from each dimension into subset and creating view from this subset
		for (int i = 1; i <= dimCount; i++)
		{
			TM1Dimension currentDim = targetCube.getDimension(i);
			elemCount = currentDim.getElementCount().getInt();
			if (currentDim.isError())
				{System.out.println("Couldn't get dimension: " + currentDim.getErrorMessage());}
			else
			{
				// Add a random number of elements into dimension subset
				int numberOfElements = 1 + r.nextInt(elemCount);
				if (debug) {System.out.println ("Creating a subset of "+ currentDim.getName().getString() +" containing " + numberOfElements + " elements");}
				
				sub = currentDim.createSubset();
				if (!sub.isError())
				{
					//if it's a restricted dimension, select a random value from given restricted elements list
					restrictedDimension = false;
					for (int l = 0; l < readRestrictedDimensions.length; l++)
					{
						if (readRestrictedDimensions[l].equals(currentDim.getName().getString()))
						{
								restrictedDimension = true;
								
								for (int j = 0; j < r.nextInt(readRestrictedDimensionElements[l].size()) + 1; j++)
								{
									if (debug) {System.out.println(currentDim.getName().getString() + " is a restricted dimension");}
									// Selecting a random Element from list
									String randomDimensionElement = readRestrictedDimensionElements[l].get(r.nextInt(readRestrictedDimensionElements[l].size()));
									if (debug) {System.out.println("Selected element " + randomDimensionElement);}
									//System.out.println("Is it in subset " + sub.getElementIndex(randomDimensionElement));
									if (sub.getElementIndex(randomDimensionElement) == 0)
									{
										TM1Element elem = currentDim.getElement(randomDimensionElement);
										sub.insertElement(elem,j);
									}
								}								
							}
					}
					// trying to get a normal dimension element
					if (!restrictedDimension)
					{
						for (int j = 0; j < numberOfElements; j++)
						{
							TM1Element elem = currentDim.getElement(r.nextInt(elemCount));
							if (!elem.isError())
							{
								//System.out.println ("Chose element " + elem.getName().getString());
								sub.insertElement(elem,j);
							}
						}
					}				
					if (i == 1)
					{
						//Row dimension
						if (debug) {System.out.println ("Row dimension");}
						rowSubsetArr.addToArray(sub);
					}
					else 
					{
						if (i == dimCount)
							{
								//Measures = Column dimension
								if (debug) {System.out.println ("Column dimension");}
								columnSubsetArr.addToArray(sub);
							}
						else
						{
							if (debug) {System.out.println ("Constructing Title page, dimension " + (i-2));}
							titleSubset[i-2] = sub;
							titleSubsetArr.addToArray(i-2 ,titleSubset[i-2]);
						}
					}
					
				}
				else
				{System.out.println("Couldn't construct subset: " + sub.getErrorMessage());}
			}
		}
		startTime = System.currentTimeMillis();
		vwToRead = targetCube.createView(titleSubsetArr,columnSubsetArr,rowSubsetArr);
		if (!vwToRead.isError())
		{
			//vwToRead.setSkipZeroes(false);
			//vwToRead.setSuppressZeroes(false);
			//vwToRead.setShowAutomatically(true);
			vwToRead.setSkipCalculatedValues(false);
			result = vwToRead.constructArray();
			TM1Val vwContents = vwToRead.getViewArrayValueByRange(1, 1, vwToRead.getNumberOfArrayRows(), vwToRead.getNumberOfArrayColumns());
			System.out.println ( "Array size " + vwContents.getArraySize());
			if (!vwContents.isError() && (vwContents.getArraySize() ==1))
			{
				for (int l = 0; l < vwContents.getArrayVal(0).getArraySize(); l++)
				{
					val = vwContents.getArrayVal(0).getArrayVal(l);
					 //Lines below are for debug printing read values, uncomment to get all read data dumped into console 
					/**
					int type = val.getType();
                    switch (type)
                    {
                        case 0:
	                		System.out.println ("Null Value");
    	        	        break;
        	            case 1:
            		        System.out.println ("Real\\Float Value: " + (new Double (val.getReal())).toString());
        	                break;
                        case 2:
	                        System.out.println ("String Value: " + val.getString().toString());
    	                    break;
                        case 3:
                    	    System.out.println ("Int Value: " + (new Integer (val.getInt())).toString());
                	        break;
	                    case 4:
    	    	            System.out.println ("Boolean value");
        		            break;
	                    case 5:
    		                System.out.println ("Object value");
	        	            break;
            	    	case 6:
            	    	    System.out.println ("Error value");
        	            	break;
	                    case 7:
	    	                System.out.println ("Array value");
    	    	            break;
            	        default:
                	        System.out.println ("Default value");
                    		break;
                    }/**/
				}
				
			}
			endTime = System.currentTimeMillis();
			if (debug) {System.out.println("Constructed a view and read " + vwToRead.getNumberOfArrayRows() * vwToRead.getNumberOfArrayColumns() + " cells in " + ((endTime - startTime) / 1000F) + " seconds." );}
			writeToLog(targetCube.getName().getString(),"Read " + vwToRead.getNumberOfArrayRows() * vwToRead.getNumberOfArrayColumns() + " cells", ((endTime - startTime) / 1000F));
			//cleaning up generated Views and Subsets
			vwToRead.destroyArray();
			vwToRead.destroy();
			columnSubsetArr.getArrayVal(0).getObject().destroy();
			rowSubsetArr.getArrayVal(0).getObject().destroy();
			for (int m = 0; m < titleSubsetArr.getArraySize(); m++ )
			{titleSubsetArr.getArrayVal(m).getObject().destroy();}
		}
		else
		{
			System.out.println("Couldn't read data from constructed view: " + vwToRead.getErrorMessage());
		}
	}
	public void writeDataToCube(TM1Cube targetCube, Random r, TM1Server conn)
	/* Write a random number of cells to the cube */
	{
		TM1Val cellCoordinates;
		TM1Element cellCoordinate;
		TM1Val cellValue;
		TM1Val ret;
		TM1Val titleSubsetArr;
		TM1Val columnSubsetArr;
		TM1Val rowSubsetArr;
		TM1Val result;
		TM1Subset titleSubset[];
		TM1Subset sub;
		TM1Subset picklistSubset;
		TM1View vwToWrite;
		TM1Dimension currentDim;
		TM1Dimension picklistDim;
		long startTime;
		long endTime;
		boolean restrictedDimension;
		double d;
		int dimCount ;
		int numberOfCellsToWrite;
		String strCellCoordinates;
		// Select a random number of cells to write
		System.out.println ("Writing to " + targetCube.getName().getString());
		if ((debug)& (targetCube.isCubePickListExists().getBoolean()) ) {System.out.println(targetCube.getName().getString() + " has picklists");}
		numberOfCellsToWrite = r.nextInt(100) + 1;
		startTime = System.currentTimeMillis();
		dimCount = targetCube.getDimensionCount().getInt();
		
		for (int cellWrite = 0; cellWrite < numberOfCellsToWrite; cellWrite++)
		{
			//Selecting Random Coordinates in cube to read
			//Future cell coordinates
			cellCoordinates =TM1Val.makeArrayVal (dimCount);
			if (debug) {System.out.println ("Writing cell number " + cellWrite);}
			rowSubsetArr = TM1Val.makeArrayVal(1);
			columnSubsetArr = TM1Val.makeArrayVal(1);
			titleSubsetArr = TM1Val.makeArrayVal(dimCount -2);
			titleSubset = new TM1Subset[dimCount - 2];
			// Pick up some values for every dimension
			for (int i = 1; i <= dimCount; i++)
			{
				currentDim = targetCube.getDimension(i);
				if (currentDim.isError())
				{System.out.println(currentDim.getErrorMessage());}
				else
				{
					if (debug) {System.out.println("Dimension: " + currentDim.getName().getString());}
					
					
					sub = currentDim.createSubset();	
					//if it's a restricted dimension, select a random value from given restricted elements list
					restrictedDimension = false;
					for (int l = 0; l < writeRestrictedDimensions.length; l++)
					{
						if (writeRestrictedDimensions[l].equals(currentDim.getName().getString()))
						{
							restrictedDimension = true;
							if (debug) {System.out.println(currentDim.getName().getString() + " is a restricted dimension");}
							// Selecting a random Element from list
							Integer randomElement = r.nextInt(writeRestrictedDimensionElements[l].size()); 
							if (debug) {System.out.println("Selected element number " + randomElement + " dimension size " + writeRestrictedDimensionElements[l].size());}
							String randomDimensionElement = writeRestrictedDimensionElements[l].get(randomElement);
							if (debug) {System.out.println("Selected Element " + randomDimensionElement);} 
							
							cellCoordinate = currentDim.getElement(randomDimensionElement);
							if (debug) {System.out.println("Cell Type " + (cellCoordinate.getElementType()));}
							if ((cellCoordinate.getElementType() == TM1ObjectType.ElementSimple) | (cellCoordinate.getElementType() == TM1ObjectType.ElementString))
							{
								//add the element to the array
								cellCoordinates.addToArray (cellCoordinate);
								if (debug) {System.out.println("Element: " + cellCoordinate.getName().getString());}
								result = sub.insertElement(cellCoordinate,1);
								if (!result.getBoolean() ) {System.out.println("Failed to add element to subset");}
								break;
							}
						}
					}
					// trying to get a normal dimension element
					if (!restrictedDimension)
					{	
						if (debug) {System.out.println(currentDim.getName().getString() + " is not a restricted dimension");}
						for (int j = 1; j > 0; j++)
						{
							cellCoordinate = currentDim.getElement (r.nextInt(currentDim.getElementCount().getInt())+1);
							if ((cellCoordinate.getElementType() == TM1ObjectType.ElementSimple) | (cellCoordinate.getElementType() == TM1ObjectType.ElementString))
							{
								//add the element to the array
								cellCoordinates.addToArray (cellCoordinate);
								if (debug)  {System.out.println("Element: " + cellCoordinate.getName().getString());}
								sub.insertElement(cellCoordinate,1);
								break;
							}
						}
					}
					
					if (i == 1)
					{
						//Row dimension
						if (debug) {System.out.println ("Row dimension. Subset size " +  sub.getElementCount() + " element is " + sub.getElement(1).getName().getString());}
						rowSubsetArr.addToArray(sub);
					}
					else 
					{
						if (i == dimCount)
							{
								//Measures = Column dimension
								if (debug) {System.out.println ("Column dimension. Subset size " +  sub.getElementCount() + " element is " 	+ sub.getElement(1).getName().getString());}
								columnSubsetArr.addToArray(sub);
							}
						else
						{
							if (debug) {System.out.println ("Constructing Title page, dimension " + (i-2) + ". Subset size " +  sub.getElementCount() + " element is " + sub.getElement(1).getName().getString());}
							titleSubset[i-2] = sub;
							titleSubsetArr.addToArray(i-2 ,titleSubset[i-2]);
						}
					}
					
				}
			}
			//Read Cell	
			cellValue = targetCube.getCellValue (cellCoordinates);
			//Writing data
			int type = cellValue.getType();
			// Need this view to detect whether it's a picklist cell
			vwToWrite = targetCube.createView(titleSubsetArr,columnSubsetArr,rowSubsetArr);
			
			
			if (!vwToWrite.isError())
			{
				result = vwToWrite.constructArray();
				if (result.getBoolean())
				{	
					TM1Val pickList = vwToWrite.getPickList(2,2);
					//cleaning up generated Views and Subsets
					vwToWrite.destroyArray();
					vwToWrite.destroy();
					columnSubsetArr.getArrayVal(0).getObject().destroy();
					rowSubsetArr.getArrayVal(0).getObject().destroy();
					for (int m = 0; m < titleSubsetArr.getArraySize(); m++ )
					{titleSubsetArr.getArrayVal(m).getObject().destroy();}
					
					if (pickList.isError())
					{	//not a picklist
						switch (type)
						{
							
							case 0:
							if (debug)  {System.out.println ("Null Value");}
							
							break;
							case 1:
								if (debug)  {System.out.println ("Writing Real\\Float Value");}
								d = r.nextInt(MAX_NUMBER_TO_WRITE);
								ret = targetCube.setCellValue(cellCoordinates, new TM1Val (d));
								if (ret.isError())
								{
									// record cell coordinates where we tried to write for choosing dimension elements
									strCellCoordinates = "";
									for (int m = 0; m < cellCoordinates.getArraySize(); m++)
									{
										strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
									}
									System.out.println("Write Error, cell " + strCellCoordinates + " : "+ ret.getErrorMessage());
								}
								
								break;
							case 2:
								//ret = targetCube.setCellValue (cellCoordinates, new TM1Val (Integer.toString(r.nextInt())));
								String str = Long.toHexString(Double.doubleToLongBits(r.nextInt()));
								if (debug)  {System.out.println ("Writing string value" + str);};
								ret = targetCube.setCellValue (cellCoordinates, new TM1Val (str));
								if (ret.isError())
								{// record cell coordinates where we tried to write for choosing dimension elements
									strCellCoordinates = "";
									for (int m = 0; m < cellCoordinates.getArraySize(); m++)
									{
										strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
									}
									System.out.println("Write Error, cell " + strCellCoordinates + " : " + ret.getErrorMessage());
								}
								if (debug)  {System.out.println ("String Value: " + cellValue.getString().toString());}
								
								break;
							case 3:
								if (debug)  {System.out.println ("Writing int Value");}
								ret = targetCube.setCellValue (cellCoordinates, new TM1Val (r.nextInt(MAX_NUMBER_TO_WRITE)));
								if (ret.isError())
								{// record cell coordinates where we tried to write for choosing dimension elements
									strCellCoordinates = "";
									for (int m = 0; m < cellCoordinates.getArraySize(); m++)
									{
										strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
									}
									System.out.println("Write Error, cell " + strCellCoordinates + " : "+ ret.getErrorMessage());
								}
								
								break;
							case 4:
								if (debug)  {System.out.println ("Boolean value");}
								
								break;
							case 5:
								if (debug)  {System.out.println ("Object value");}
								
								break;
							case 6:
								if (debug)  {System.out.println ("Error value");}
								
								break;
							case 7:
								if (debug)  {System.out.println ("Array value");}
								
								break;
							default:
								if (debug)  {System.out.println ("Default value");}
								
								break;
							/**/
						}
					}
					else
					{
						if (debug)  {System.out.println("This is a picklist cell");}
						switch (pickList.getArrayVal(0).getInt())
						{
						
							case 1:
							{
								// Static choice picklist -- all piclist variants are included in second part of array
								 String picklistElement = pickList.getArrayVal(1).getArrayVal(r.nextInt(pickList.getArrayVal(1).getArraySize())).getString();
								 if (debug) { System.out.println("selected element for picklist" + picklistElement);}
								 ret = targetCube.setCellValue (cellCoordinates, new TM1Val (picklistElement));
								if (ret.isError())
								{// record cell coordinates where we tried to write for choosing dimension elements
									strCellCoordinates = "";
									for (int m = 0; m < cellCoordinates.getArraySize(); m++)
									{
										strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
									}
									System.out.println("Write Error, picklist cell " + strCellCoordinates + " : " + ret.getErrorMessage());
								}
								
								break;
							}
							case 2:
							{// Dimension subset Picklist. First part of array is dimension name, second -- subset name
								picklistDim = conn.getDimension(pickList.getArrayVal(1).getArrayVal(0).getString());
								if (!picklistDim.isError())
								{
									// if subset is empty -- skipping filling -- this is a conditional picklist
									if (!pickList.getArrayVal(1).getArrayVal(1).isError())
									{
										picklistSubset = picklistDim.getSubset(pickList.getArrayVal(1).getArrayVal(1).getString());
										if (!picklistSubset.isError())
										{
											if (debug) { System.out.println("Got subset ") ;} 
											int randomElementIndex = r.nextInt(picklistSubset.getElementCount()) + 1;
											if (debug) { System.out.println("Subset size " + picklistSubset.getElementCount() + " selected element number " + randomElementIndex);}	
											String picklistElement = picklistSubset.getElement(randomElementIndex).getName().getString();
											if (debug) { System.out.println("selected element for picklist" + picklistElement);}	
											ret = targetCube.setCellValue (cellCoordinates, new TM1Val (picklistElement));
											if (ret.isError())
											{// record cell coordinates where we tried to write for choosing dimension elements
												strCellCoordinates = "";
												for (int m = 0; m < cellCoordinates.getArraySize(); m++)
												{
													strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
												}
												System.out.println("Write Error, picklist cell " + strCellCoordinates + " : " + ret.getErrorMessage());
											}
										}
										else
										{System.out.println("Coludn't select subset " + picklistDim.getSubset(pickList.getArrayVal(1).getArrayVal(1).getString()) +  " for dimension " + pickList.getArrayVal(1).getArrayVal(9).getString() + " for picklist") ;}	
									}
								}
								else
								{System.out.println("Coludn't select dimension " + pickList.getArrayVal(1).getArrayVal(0).getString() + " for picklist") ;}
								break;
							}
							case 3:
							{// Dimension picklist — just a dimension name as a second argument, without subset
								picklistDim = conn.getDimension(pickList.getArrayVal(1).getArrayVal(0).getString());
								if (!picklistDim.isError())
								{
									
									int randomElementIndex = r.nextInt(picklistDim.getElementCount().getInt()) + 1;
									if (debug) { System.out.println("Dimension size " + picklistDim.getElementCount().getInt() + " selected element number " + randomElementIndex);}	
									String picklistElement = picklistDim.getElement(randomElementIndex).getName().getString();
									if (debug) { System.out.println("selected element for picklist" + picklistElement);}	
									ret = targetCube.setCellValue (cellCoordinates, new TM1Val (picklistElement));
									if (ret.isError())
									{// record cell coordinates where we tried to write for choosing dimension elements
										strCellCoordinates = "";
										for (int m = 0; m < cellCoordinates.getArraySize(); m++)
										{
											strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
										}
										System.out.println("Write Error, picklist cell " + strCellCoordinates + " : " + ret.getErrorMessage());
									}
								}
								else
								{System.out.println("Coludn't select dimension " + pickList.getArrayVal(1).getArrayVal(0).getString() + " for picklist") ;}
								
								break;
							}
							case 4:
							{ // this a false detected picklist -- it's just a normal cell for input. It's just mentioned in }picklistCube
								
								switch (type)
								{
									
									case 1:
										if (debug)  {System.out.println ("Writing Real\\Float Value");}
										d = r.nextInt(MAX_NUMBER_TO_WRITE);
										ret = targetCube.setCellValue(cellCoordinates, new TM1Val (d));
										if (ret.isError())
										{
											// record cell coordinates where we tried to write for choosing dimension elements
											strCellCoordinates = "";
											for (int m = 0; m < cellCoordinates.getArraySize(); m++)
											{
												strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
											}
											System.out.println("Write Error, cell " + strCellCoordinates + " : "+ ret.getErrorMessage());
										}
										break;
									case 2:
										//ret = targetCube.setCellValue (cellCoordinates, new TM1Val (Integer.toString(r.nextInt())));
										String str = Long.toHexString(Double.doubleToLongBits(r.nextInt()));
										if (debug)  {System.out.println ("Writing string value" + str);};
										ret = targetCube.setCellValue (cellCoordinates, new TM1Val (str));
										if (ret.isError())
										{// record cell coordinates where we tried to write for choosing dimension elements
											strCellCoordinates = "";
											for (int m = 0; m < cellCoordinates.getArraySize(); m++)
											{
												strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
											}
											System.out.println("Write Error, cell " + strCellCoordinates + " : " + ret.getErrorMessage());
										}
										if (debug)  {System.out.println ("String Value: " + cellValue.getString().toString());}
										break;
									case 3:
										if (debug)  {System.out.println ("Writing int Value");}
										ret = targetCube.setCellValue (cellCoordinates, new TM1Val (r.nextInt(MAX_NUMBER_TO_WRITE)));
										if (ret.isError())
										{// record cell coordinates where we tried to write for choosing dimension elements
											strCellCoordinates = "";
											for (int m = 0; m < cellCoordinates.getArraySize(); m++)
											{
												strCellCoordinates += cellCoordinates.getArrayVal(m).getObject().getName().getString() + ";";
											}
											System.out.println("Write Error, cell " + strCellCoordinates + " : "+ ret.getErrorMessage());
										}
										break;
								}
								break;
							
							}
							default:
							{
								System.out.println("Unknown picklist type -- add more picklist types"); 
								System.out.println("Array size " + pickList.getArraySize() + " . First Element (type) is " +  pickList.getArrayVal(0).getInt());
								System.out.println("Array size " + pickList.getArraySize() + " . First Element (type) is " +  pickList.getArrayVal(0).getInt()  
								+ " Second element type is " + pickList.getArrayVal(1).getType() + " and it's length is " + pickList.getArrayVal(1).getArraySize() 
								+  ". It's header  " +  pickList.getArrayVal(1).getArrayVal(0).getString() 
								+ ". Followed by  " +  pickList.getArrayVal(1).getArrayVal(1).getString()
								);
								break;
							}
						}
						
					}	
					
				}	
					
				else 
				{System.out.println("Couldn't construct write view " + result.getErrorMessage());}
				
			}
			else
			{System.out.println("Couldn't create write View " + vwToWrite.getErrorMessage());}
		}
		endTime = System.currentTimeMillis();
		writeToLog(targetCube.getName().getString(),"Wrote " + numberOfCellsToWrite + " cells", ((endTime - startTime) / 1000F));
	}
	
	public void run()
	{
		try 
		{
			TM1Bean fieldAdminServer;
			TM1Server conn;
			fieldAdminServer = new TM1Bean();
			fieldAdminServer.setAdminHost(AdminHost);
			// trying to connect
			conn = fieldAdminServer.openConnection1001(TM1ServerName, user, password);
			if (conn.isError() )
			{
				System.out.println (conn.getErrorMessage());
			}
			else
			{	
				System.out.println ("Logged in as " + user);
				Random r = new Random();
				TM1Cube targetCube;
				int cubeIndex;
				
				for (int step = 0; step >= 0; step ++)
				{
					if (debug) {System.out.println ("Deciding whether to read or write");}	
					// flipping a coin to read or write
					if (r.nextInt(2) == 0)
					{ // read data
						cubeIndex = r.nextInt(cubesToRead.length);
						if (debug) {System.out.println ("Reading data from cube " + cubesToRead[cubeIndex]);}	
						targetCube = conn.getCube(cubesToRead[cubeIndex]);
						if (targetCube.isError())
						{
							System.out.println (targetCube.getErrorMessage());
						}
						else
						{
							readDataFromCube (targetCube, r, conn);
						}
					}
					else
					{
						if (cubesToWrite.length > 0)
						{
							//write data
							cubeIndex = r.nextInt(cubesToWrite.length);
							targetCube = conn.getCube(cubesToWrite[cubeIndex]);
							if (targetCube.isError())
							{
								System.out.println (targetCube.getErrorMessage());
							}
							else
							{
								writeDataToCube (targetCube, r, conn);
							}
						}
					}
					Thread.currentThread().sleep(r.nextInt(secondsBetweenActions) * 1000);
				}
			}
		}
		catch(Exception e)
		{
			System.out.println(e);
		}
	}
}
public class tm1_stress_test
{
public static void main(String[] args) {
	/* reading user configuration from file and running multiple sessions*/
	try 
	{
		if (args.length == 0)
		{
			System.out.println ("Please supply a valid xml profile file.");
		}
		else
		{
			File file = new File(args[0]);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document conf = db.parse(file);
			conf.getDocumentElement().normalize();
			//parsing configuration
			boolean debug = Boolean.parseBoolean(conf.getElementsByTagName("debug").item(0).getChildNodes().item(0).getNodeValue());
			int numberOfSessions = new Integer(conf.getElementsByTagName("number_of_sessions").item(0).getChildNodes().item(0).getNodeValue()).intValue();
			String AdminHost = conf.getElementsByTagName("AdminHost").item(0).getChildNodes().item(0).getNodeValue();
			String TM1ServerName = conf.getElementsByTagName("tm1server").item(0).getChildNodes().item(0).getNodeValue();
			int secondsBetweenActions = new Integer(conf.getElementsByTagName("seconds_between_actions").item(0).getChildNodes().item(0).getNodeValue()).intValue();
			// users and passwords
			NodeList userLogins = conf.getElementsByTagName("user_login");
			String[] users = new String[userLogins.getLength()];
			String[] passwords = new String[userLogins.getLength()];
			for (int i = 0; i < userLogins.getLength(); i++) 
			{
				Node curr = userLogins.item(i);
				if (curr.getNodeType() ==  Node.ELEMENT_NODE)
				{
					Element fstElmnt = (Element) curr;
					users[i] = fstElmnt.getElementsByTagName("login").item(0).getChildNodes().item(0).getNodeValue();
					if (fstElmnt.getElementsByTagName("password").item(0).getChildNodes().item(0) == null)
						{passwords[i] = "";}
					else 
						{passwords[i] = fstElmnt.getElementsByTagName("password").item(0).getChildNodes().item(0).getNodeValue();}
					//System.out.println (users[i] + passwords[i]);
				}
			}
			//cubes lists
			NodeList cubesToWriteLst = conf.getElementsByTagName("cube_to_write");
			String[] cubesToWrite = new String[cubesToWriteLst.getLength()];
			for (int i = 0; i < cubesToWriteLst.getLength(); i++) 
			{
				cubesToWrite[i] = cubesToWriteLst.item(i).getChildNodes().item(0).getNodeValue(); 
				if (debug) {System.out.println ("Adding cube " + cubesToWrite[i] + " to cubes_to_write list");}	
			}
			NodeList cubesToReadLst = conf.getElementsByTagName("cube_to_read");
			String[] cubesToRead = new String[cubesToReadLst.getLength() + cubesToWrite.length];
			for (int i = 0; i < cubesToReadLst.getLength(); i++) 
			{
				cubesToRead[i] = cubesToReadLst.item(i).getChildNodes().item(0).getNodeValue();
				if (debug) {System.out.println ("Adding cube " + cubesToRead[i] + " to cubes_to_read list");}	
				 
			}
			for (int m = cubesToReadLst.getLength(); m < cubesToRead.length; m++)
			{
				cubesToRead[m] = cubesToWrite [m - cubesToReadLst.getLength()];
				if (debug) {System.out.println ("Copying cube " + cubesToWrite [m - cubesToReadLst.getLength()] + " from cubes_to_write to cubes_to_read list");}	
			}
			//Write Restricted dimensions
			NodeList writeDimensionElementsList = conf.getElementsByTagName("write_dimension_elements");
			String[] writeRestrictedDimensions = new String[writeDimensionElementsList.getLength()];
			ArrayList<String>[] writeRestrictedDimensionElements = (ArrayList<String>[])new ArrayList[writeDimensionElementsList.getLength()];
			for (int i = 0; i < writeDimensionElementsList.getLength(); i++) 
			{
				Node curr = writeDimensionElementsList.item(i);
				if (curr.getNodeType() ==  Node.ELEMENT_NODE)
				{
					Element fstElmnt = (Element) curr;
					writeRestrictedDimensions[i] = fstElmnt.getElementsByTagName("write_dimension").item(0).getChildNodes().item(0).getNodeValue();
					//adding elements to arraylist
					NodeList elemList = fstElmnt.getElementsByTagName("write_element");
					ArrayList dimElements = new ArrayList();
					for (int j = 0;j < elemList.getLength(); j++)
					{
						String element = elemList.item(j).getChildNodes().item(0).getNodeValue();
						dimElements.add(element);
						writeRestrictedDimensionElements[i] = dimElements;
					}
				}
			}
			//Read Restricted dimensions
			NodeList readDimensionElementsList = conf.getElementsByTagName("read_dimension_elements");
			String[] readRestrictedDimensions = new String[readDimensionElementsList.getLength()];
			ArrayList<String>[] readRestrictedDimensionElements = (ArrayList<String>[])new ArrayList[readDimensionElementsList.getLength()];
			for (int i = 0; i < readDimensionElementsList.getLength(); i++) 
			{
				Node curr = readDimensionElementsList.item(i);
				if (curr.getNodeType() ==  Node.ELEMENT_NODE)
				{
					Element fstElmnt = (Element) curr;
					readRestrictedDimensions[i] = fstElmnt.getElementsByTagName("read_dimension").item(0).getChildNodes().item(0).getNodeValue();
					//adding elements to arraylist
					NodeList elemList = fstElmnt.getElementsByTagName("read_element");
					ArrayList dimElements = new ArrayList();
					for (int j = 0;j < elemList.getLength(); j++)
					{
						String element = elemList.item(j).getChildNodes().item(0).getNodeValue();
						dimElements.add(element);
						readRestrictedDimensionElements[i] = dimElements;
					}
				}
			}
			/**
			// Checking restricted dimensions filling
			for (int i = 0; i < readRestrictedDimensions.length; i++)
			{
				System.out.println ("For read dimension " + readRestrictedDimensions[i] + " elements are ");
				for (int j = 0; j < readRestrictedDimensionElements[i].size(); j++)
				{
					System.out.println("\t" + readRestrictedDimensionElements[i].get(j));
				}
			}
			/**/
			// Log file
			FileWriter fstream = new FileWriter("test_results_" + args[0].replace(".xml","")  + ".csv");
			BufferedWriter logFile = new BufferedWriter(fstream);
			logFile.write("SessionId;UserName;Time;Cube;Operation;Duration;\n");
			// adding a hook to close logging
			Runtime rt = Runtime.getRuntime();
			rt.addShutdownHook(new CloseLogging(logFile));
			
			// Establishing connection with TM1 server and spawning needed sessions
			TM1Bean fieldAdminServer;
			TM1Server fieldServer;

			fieldAdminServer = new TM1Bean();
			fieldAdminServer.setAdminHost(AdminHost);
			// trying to connect
			fieldServer = fieldAdminServer.openConnection(TM1ServerName, users[0], passwords[0]);
			if (fieldServer.isError() )
			{
				System.out.println ("Couldn't connect to Admin Host: " + fieldServer.getErrorMessage());
			}
			else
			{
				System.out.println ("Running tests");
				
				//spawn user sessions
				for (int i = 0; i < numberOfSessions; i++)
				{	//selecting user and password
					int index = i%users.length;
					Thread user = new UserThread(AdminHost,TM1ServerName, users[index], passwords[index], cubesToRead, cubesToWrite, writeRestrictedDimensions, writeRestrictedDimensionElements, readRestrictedDimensions, readRestrictedDimensionElements, logFile, i, debug, secondsBetweenActions);
					user.start();
				}
				
			}
		}
	}
	catch (Exception e) 
	{
		e.printStackTrace();
	}
}

}
